Index: src/control/media_instance.c
===================================================================
--- src/control/media_instance.c	(revision 22322)
+++ src/control/media_instance.c	(working copy)
@@ -430,10 +430,12 @@
     if( (p_input_thread = libvlc_get_input_thread( p_mi, p_e )) )
     {
         /* A thread alread exists, send it a play message */
-        vlc_value_t val;
-        val.i_int = PLAYING_S;
+        // vlc_value_t val;
+        // val.i_int = PLAYING_S;
+        // 
+        // input_Control( p_input_thread, INPUT_CONTROL_SET_STATE, PLAYING_S );
+        input_ChangeState( p_input_thread, PLAYING_S );
 
-        input_Control( p_input_thread, INPUT_CONTROL_SET_STATE, PLAYING_S );
         vlc_object_release( p_input_thread );
         return;
     }
@@ -477,15 +479,17 @@
                                   libvlc_exception_t *p_e )
 {
     input_thread_t * p_input_thread;
-    vlc_value_t val;
-    val.i_int = PAUSE_S;
+    // vlc_value_t val;
+    // val.i_int = PAUSE_S;
 
     p_input_thread = libvlc_get_input_thread( p_mi, p_e );
 
     if( !p_input_thread )
         return;
 
-    input_Control( p_input_thread, INPUT_CONTROL_SET_STATE, val );
+    //input_Control( p_input_thread, INPUT_CONTROL_SET_STATE, val );
+    input_ChangeState( p_input_thread, PAUSE_S );
+
     vlc_object_release( p_input_thread );
 }
 
Index: src/input/input.c
===================================================================
--- src/input/input.c	(revision 22322)
+++ src/input/input.c	(working copy)
@@ -1592,9 +1592,22 @@
         }
 
         case INPUT_CONTROL_SET_STATE:
-            if( ( val.i_int == PLAYING_S && p_input->i_state == PAUSE_S ) ||
-                ( val.i_int == PAUSE_S && p_input->i_state == PAUSE_S ) )
+            if( ( val.i_int == p_input->i_state ) )
             {
+                // Nothing to do, new state == old state
+                break;
+            }
+            else if( ( val.i_int == PLAYING_S && p_input->i_state == OPENING_S ) ||
+                     ( val.i_int == PLAYING_S && p_input->i_state == BUFFERING_S ) )
+            {
+                // If the video was directed to play from the state of buffering
+                // then update the input with the new state, the play will start 
+                // elsewhere
+                p_input->i_state = val.i_int;
+            }
+            else if( ( val.i_int == PLAYING_S && p_input->i_state == PAUSE_S ) ||
+                     ( val.i_int == PAUSE_S && p_input->i_state == PAUSE_S ) )
+            {
                 int i_ret;
                 if( p_input->p->input.p_access )
                     i_ret = access2_Control( p_input->p->input.p_access,
@@ -1663,8 +1676,12 @@
             }
             else if( val.i_int != PLAYING_S && val.i_int != PAUSE_S )
             {
-                msg_Err( p_input, "invalid state in INPUT_CONTROL_SET_STATE" );
+                msg_Err( p_input, "invalid state in INPUT_CONTROL_SET_STATE \"%d\"", val.i_int );
             }
+            else
+            {
+                msg_Dbg( p_input, "current state in INPUT_CONTROL_SET_STATE \"%d\" but didn't know what to do with new state \"%d\"", p_input->i_state, val.i_int );
+            }
             break;
 
         case INPUT_CONTROL_SET_RATE:
Index: src/input/input_internal.h
===================================================================
--- src/input/input_internal.h	(revision 22322)
+++ src/input/input_internal.h	(working copy)
@@ -310,7 +310,8 @@
 
 static inline void input_ChangeState( input_thread_t *p_input, int state )
 {
-    var_SetInteger( p_input, "state", p_input->i_state = state );
+    // var_SetInteger( p_input, "state", p_input->i_state = state );
+    var_SetInteger( p_input, "state", state );
 }
 
 /* Access */
Index: src/input/var.c
===================================================================
--- src/input/var.c	(revision 22322)
+++ src/input/var.c	(working copy)
@@ -483,6 +483,12 @@
         input_ControlPush( p_input, INPUT_CONTROL_SET_STATE, &newval );
         return VLC_SUCCESS;
     }
+    else
+    {
+        // If the state change was not a play or a pause, then we need to manually
+        // update the p_input with the new state
+        p_input->i_state = newval.i_int;    
+    }
 
     return VLC_EGENERIC;
 }
